import os
import time
import asyncio

from typing import Optional
import multiprocessing
from queue import Empty

from warp_beacon.scraper.exceptions import NotFound, UnknownError, TimeOut, Unavailable, FileTooBig, YotubeLiveError, YotubeAgeRestrictedError
from warp_beacon.mediainfo.video import VideoInfo
from warp_beacon.mediainfo.audio import AudioInfo
from warp_beacon.mediainfo.silencer import Silencer
from warp_beacon.compress.video import VideoCompress
from warp_beacon.uploader import AsyncUploader
from warp_beacon.jobs import Origin
from warp_beacon.jobs.download_job import DownloadJob
from warp_beacon.jobs.upload_job import UploadJob
from warp_beacon.jobs.types import JobType

import logging

class AsyncDownloader(object):
	__JOE_BIDEN_WAKEUP = None
	workers = []
	allow_loop = None
	job_queue = multiprocessing.Queue()
	uploader = None
	workers_count = 0
	auth_event = multiprocessing.Event()

	def __init__(self, uploader: AsyncUploader, workers_count: int) -> None:
		self.allow_loop = multiprocessing.Value('i', 1)
		self.uploader = uploader
		self.workers_count = workers_count

	def __del__(self) -> None:
		self.stop_all()

	def start(self) -> None:
		for _ in range(self.workers_count):
			proc = multiprocessing.Process(target=self.do_work)
			self.workers.append(proc)
			proc.start()

	def get_media_info(self, path: str, fr_media_info: dict={}, media_type: JobType = JobType.VIDEO) -> Optional[dict]:
		media_info = None
		try:
			if path:
				if media_type == JobType.VIDEO:
					video_info = VideoInfo(path)
					media_info = video_info.get_finfo(tuple(fr_media_info.keys()))
					media_info.update(fr_media_info)
					if not media_info.get("thumb", None):
						media_info["thumb"] = video_info.generate_thumbnail()
					media_info["has_sound"] = video_info.has_sound()
				elif media_type == JobType.AUDIO:
					audio_info = AudioInfo(path)
					media_info = audio_info.get_finfo(tuple(fr_media_info.keys()))
		except Exception as e:
			logging.error("Failed to process media info!")
			logging.exception(e)

		return media_info

	def do_work(self) -> None:
		logging.info("download worker started")
		while self.allow_loop.value == 1:
			try:
				job = None
				try:
					job = self.job_queue.get()
					if job is self.__JOE_BIDEN_WAKEUP:
						continue
					actor = None
					try:
						items = []
						if job.job_origin is not Origin.UNKNOWN:
							if not job.in_process:
								actor = None
								if job.job_origin is Origin.INSTAGRAM:
									from warp_beacon.scraper.instagram import InstagramScraper
									actor = InstagramScraper()
								elif job.job_origin is Origin.YT_SHORTS:
									from warp_beacon.scraper.youtube.shorts import YoutubeShortsScraper
									actor = YoutubeShortsScraper()
								elif job.job_origin is Origin.YT_MUSIC:
									from warp_beacon.scraper.youtube.music import YoutubeMusicScraper
									actor = YoutubeMusicScraper()
								elif job.job_origin is Origin.YOUTUBE:
									from warp_beacon.scraper.youtube.youtube import YoutubeScraper
									actor = YoutubeScraper()
								#self.auth_event = multiprocessing.Event()
								actor.send_message_to_admin_func = self.send_message_to_admin
								actor.auth_event = self.auth_event
								while True:
									try:
										logging.info("Downloading URL '%s'", job.url)
										items = actor.download(job.url)
										break
									except (NotFound, Unavailable) as e:
										logging.warning("Not found or unavailable error occurred!")
										logging.exception(e)
										self.uploader.queue_task(job.to_upload_job(
											job_failed=True,
											job_failed_msg="Unable to access to media under this URL. Seems like the media is private.")
										)
										break
									except TimeOut as e:
										logging.warning("Timeout error occurred!")
										logging.exception(e)
										self.uploader.queue_task(job.to_upload_job(
											job_failed=True,
											job_failed_msg="Failed to download content. Please check you Internet connection or retry amount bot configuration settings.")
										)
										break
									except FileTooBig as e:
										logging.warning("Telegram limits exceeded :(")
										logging.exception(e)
										self.uploader.queue_task(job.to_upload_job(
											job_failed=True,
											job_failed_msg="Unfortunately this file has exceeded the Telegram limits. A file cannot be larger than 2 gigabytes.")
										)
										break
									except YotubeLiveError as e:
										logging.warning("Youtube Live videos are not supported. Skipping.")
										logging.exception(e)
										self.uploader.queue_task(job.to_upload_job(
											job_failed=True,
											job_failed_msg="Youtube Live videos are not supported. Please wait until the live broadcast ends.")
										)
										break
									except YotubeAgeRestrictedError as e:
										logging.error("Youtube Age Restricted error")
										logging.exception(e)
										self.uploader.queue_task(job.to_upload_job(
											job_failed=True,
											job_failed_msg="Youtube Age Restricted error. Check your bot Youtube account settings.")
										)
										break
									except (UnknownError, Exception) as e:
										logging.warning("UnknownError occurred!")
										logging.exception(e)
										exception_msg = ""
										if hasattr(e, "message"):
											exception_msg = e.message
										else:
											exception_msg = str(e)
										if "geoblock_required" in exception_msg:
											self.uploader.queue_task(job.to_upload_job(
												job_failed=True,
												job_failed_msg="This content does not accessible for bot account. Seems like author blocked certain region.")
											)
											break
										self.uploader.queue_task(job.to_upload_job(
											job_failed=True,
											job_failed_msg="WOW, unknown error occured! Please [create issue](https://github.com/sb0y/warp_beacon/issues) with service logs.")
										)
										break

								if items:
									for item in items:
										media_info = {"filesize": 0}
										if item["media_type"] == JobType.VIDEO:
											media_info = self.get_media_info(item["local_media_path"], item.get("media_info", {}), JobType.VIDEO)
											logging.info("Final media info: %s", media_info)
											if media_info["filesize"] > 2e+9:
												logging.info("Filesize is '%d' MiB", round(media_info["filesize"] / 1024 / 1024))
												logging.info("Detected big file. Starting compressing with ffmpeg ...")
												self.uploader.queue_task(job.to_upload_job(
													job_warning=True,
													job_warning_msg="Downloaded file size is bigger than Telegram limits! Performing video compression. This may take a while.")
												)
												ffmpeg = VideoCompress(file_path=item["local_media_path"])
												new_filepath = ffmpeg.generate_filepath(base_filepath=item["local_media_path"])
												if ffmpeg.compress_to(new_filepath, target_size=2000 * 1000):
													logging.info("Successfully compressed file '%s'", new_filepath)
													os.unlink(item["local_media_path"])
													item["local_media_path"] = new_filepath
													item["local_compressed_media_path"] = new_filepath
													media_info["filesize"] = VideoInfo.get_filesize(new_filepath)
													logging.info("New file size of compressed file is '%.3f'", media_info["filesize"])
											if not media_info["has_sound"]:
												item["media_type"] = JobType.ANIMATION
										elif item["media_type"] == JobType.AUDIO:
											media_info = self.get_media_info(item["local_media_path"], item.get("media_info", {}), JobType.AUDIO)
											media_info["performer"] = item.get("performer", None)
											media_info["thumb"] = item.get("thumb", None)
											logging.info("Final media info: %s", media_info)
										elif item["media_type"] == JobType.COLLECTION:
											for chunk in item["items"]:
												for v in chunk:
													if v["media_type"] == JobType.VIDEO:
														col_media_info = self.get_media_info(v["local_media_path"], v["media_info"])
														media_info["filesize"] += int(col_media_info.get("filesize", 0))
														v["media_info"] = col_media_info
														if not v["media_info"]["has_sound"]:
															silencer = Silencer(v["local_media_path"])
															silent_video_path = silencer.add_silent_audio()
															os.unlink(v["local_media_path"])
															v["local_media_path"] = silent_video_path
															v["media_info"].update(silencer.get_finfo())
															v["media_info"]["has_sound"] = True

										job_args = {"media_type": item["media_type"], "media_info": media_info}
										if item["media_type"] == JobType.COLLECTION:
											job_args["media_collection"] = item["items"]
											if item.get("save_items", None) is not None:
												job_args["save_items"] = item.get("save_items", False)
										else:
											job_args["local_media_path"] = item["local_media_path"]
											if item.get("local_compressed_media_path", None):
												job_args["local_media_path"] = item.get("local_compressed_media_path", None)

										job_args["canonical_name"] = item.get("canonical_name", "")

										logging.debug("local_media_path: '%s'", job_args.get("local_media_path", ""))
										logging.debug("media_collection: '%s'", str(job_args.get("media_collection", {})))
										#logging.info(job_args)
										upload_job = job.to_upload_job(**job_args)
										if upload_job.is_empty():
											logging.info("Upload job is empty. Nothing to do here!")
											self.uploader.queue_task(job.to_upload_job(
												job_failed=True,
												job_failed_msg="Seems like this link doesn't contains any media.")
											)
										else:
											self.uploader.queue_task(upload_job)
							else:
								logging.info("Job already in work in parallel worker. Redirecting job to upload worker.")
								self.uploader.queue_task(job.to_upload_job())
					except Exception as e:
						logging.error("Error inside download worker!")
						logging.exception(e)
						self.notify_task_failed(job)
				except Empty:
					pass
			except Exception as e:
				logging.error("Exception occurred inside worker!")
				logging.exception(e)

		logging.info("Process done")

	def stop_all(self) -> None:
		self.allow_loop.value = 0
		for proc in self.workers:
			if proc.is_alive():
				logging.info("stopping process #%d", proc.pid)
				self.job_queue.put_nowait(self.__JOE_BIDEN_WAKEUP)
				proc.join()
				#proc.terminate()
				logging.info("process #%d stopped", proc.pid)
		self.workers.clear()

	def queue_task(self, job: DownloadJob) -> str:
		self.job_queue.put_nowait(job)
		return str(job.job_id)
	
	def notify_task_failed(self, job: DownloadJob) -> None:
		self.uploader.queue_task(job.to_upload_job(job_failed=True))

	def send_message_to_admin(self, text: str) -> None:
		self.uploader.queue_task(UploadJob.build(
			is_message_to_admin=True,
			message_text=text
		))


import os
import threading
import multiprocessing
from warp_beacon.jobs.upload_job import UploadJob
#import time
import logging

import asyncio

from typing import Optional, Callable, Coroutine

from warp_beacon.storage import Storage
from warp_beacon.jobs.types import JobType

class AsyncUploader(object):
	__JOE_BIDEN_WAKEUP = None
	threads = []
	allow_loop = True
	job_queue = None
	callbacks = {}
	storage = None
	in_process = set()
	loop = None
	admin_message_callback = None
	pool_size = 1

	def __init__(self, loop: asyncio.AbstractEventLoop, storage: Storage, admin_message_callback: Callable, pool_size: int=min(32, os.cpu_count() + 4)) -> None:
		self.storage = storage
		self.loop = loop
		self.job_queue = multiprocessing.Queue()
		self.admin_message_callback = admin_message_callback
		self.pool_size = pool_size
	
	def __del__(self) -> None:
		self.stop_all()

	def start(self) -> None:
		for _ in range(self.pool_size):
			thread = threading.Thread(target=self.do_work)
			thread.start()
			self.threads.append(thread)

	def add_callback(self, message_id: int, callback: Callable) -> None:
		def callback_wrap(*args, **kwargs) -> None:
			ret = callback(*args, **kwargs)
			#self.remove_callback(message_id)
			return ret
		self.callbacks[message_id] = {"callback": callback_wrap}

	def remove_callback(self, message_id: int) -> None:
		if message_id in self.callbacks:
			logging.debug("Removing callback with message id #%d", message_id)
			del self.callbacks[message_id]

	def stop_all(self) -> None:
		self.allow_loop = False
		if self.threads:
			for i in self.threads:
				t_id = i.native_id
				logging.info("Stopping thread #'%s'", t_id)
				self.job_queue.put(self.__JOE_BIDEN_WAKEUP)
				i.join()
				logging.info("Thread #'%s' stopped", t_id)
		self.threads.clear()

	def is_inprocess(self, uniq_id: str) -> bool:
		return uniq_id in self.in_process
	
	def process_done(self, uniq_id: str) -> None:
		self.in_process.discard(uniq_id)

	def set_inprocess(self, uniq_id: str) -> None:
		self.in_process.add(uniq_id)

	def queue_task(self, job: UploadJob) -> None:
		self.job_queue.put_nowait(job)

	def do_work(self) -> None:
		logging.info("Upload worker started")
		while self.allow_loop:
			try:
				try:
					job = self.job_queue.get()
					if job is self.__JOE_BIDEN_WAKEUP:
						continue
					if job.is_message_to_admin and job.message_text and self.admin_message_callback:
						asyncio.ensure_future(self.admin_message_callback(job.message_text), loop=self.loop)
						continue
					path = ""
					if job.media_type == JobType.COLLECTION:
						for i in job.media_collection:
							for j in i:
								path += "%s; " % j.local_media_path
					else:
						path = job.local_media_path
					in_process = job.in_process
					uniq_id = job.uniq_id
					message_id = job.placeholder_message_id
					if not in_process and not job.job_failed and not job.job_warning:
						logging.info("Accepted upload job, file(s): '%s'", path)
					try:
						if message_id in self.callbacks:
							if job.job_failed:
								logging.info("URL '%s' download failed. Skipping upload job ...", job.url)
								if job.job_failed_msg: # we want to say something to user
									asyncio.ensure_future(self.callbacks[message_id]["callback"](job), loop=self.loop)
								self.process_done(uniq_id)
								self.remove_callback(message_id)
								continue
							if job.job_warning:
								logging.info("Job warning occurred ...")
								if job.job_warning_msg:
									asyncio.ensure_future(self.callbacks[message_id]["callback"](job), loop=self.loop)
								continue
							if in_process:
								db_list_dicts = self.storage.db_lookup_id(uniq_id)
								if db_list_dicts:
									tg_file_ids = [i["tg_file_id"] for i in db_list_dicts]
									dlds_len = len(db_list_dicts)
									if dlds_len > 1:
										job.tg_file_id = ",".join(tg_file_ids)
										job.media_type = JobType.COLLECTION
									elif dlds_len:
										job.tg_file_id = ",".join(tg_file_ids)
										job.media_type = JobType[db_list_dicts.pop()["media_type"].upper()]
									asyncio.ensure_future(self.callbacks[message_id]["callback"](job), loop=self.loop)
									self.process_done(uniq_id)
									self.remove_callback(message_id)
								else:
									self.queue_task(job)
							else:
								asyncio.ensure_future(self.callbacks[message_id]["callback"](job), loop=self.loop)
								self.process_done(uniq_id)
								self.remove_callback(message_id)
						else:
							logging.info("No callback no call!!")
					except Exception as e:
						logging.exception(e)
				except multiprocessing.Queue.empty:
					pass
			except Exception as e:
				logging.error("Exception occurred inside upload worker!")
				logging.exception(e)
		logging.info("Thread done")



import os
#from typing import Optional
from enum import Enum

from urllib.parse import urlparse, parse_qs

from pymongo import MongoClient

import logging

class UrlParseMode(Enum):
	OTHER = 0
	YT_MUSIC = 1
	YT_SHORTS = 2
	YOUTUBE = 3

VIDEO_STORAGE_DIR = os.environ.get("VIDEO_STORAGE_DIR", default="/var/warp_beacon/videos")

class Storage(object):
	client = None
	db = None
	def __init__(self) -> None:
		if not os.path.isdir(VIDEO_STORAGE_DIR):
			os.mkdir(VIDEO_STORAGE_DIR)

		self.client = MongoClient(
			host=os.environ.get("MONGODB_HOST", default='127.0.0.1'), 
			port=int(os.environ.get("MONGODB_PORT", default=27017)),
			username=os.environ.get("MONGODB_USER", default='root'),
			password=os.environ.get("MONGODB_PASSWORD", default="changeme"))
		self.db = self.client.media.media

	def __del__(self) -> None:
		if self.client:
			self.client.close()

	@staticmethod
	def compute_uniq(url: str) -> str:
		parse_mode = UrlParseMode.OTHER
		if "music.youtube.com/" in url:
			parse_mode = UrlParseMode.YT_MUSIC
		elif "youtube.com/shorts/" in url:
			parse_mode = UrlParseMode.YT_SHORTS
		elif "youtube.com/" in url:
			parse_mode = UrlParseMode.YOUTUBE

		if parse_mode is not UrlParseMode.OTHER and parse_mode is not UrlParseMode.YT_SHORTS:
			purl = urlparse(url)
			qs = parse_qs(purl.query)
			yt_vid_id_list = qs.get('v', None)
			yt_vid_id = yt_vid_id_list.pop() if yt_vid_id_list else ""
			if yt_vid_id:
				path = urlparse(url).path.strip('/').replace("watch", ("yt_music" if parse_mode is UrlParseMode.YT_MUSIC else "youtube"))
				return ("%s/%s" % (path, yt_vid_id)).strip('/')
			else:
				raise ValueError("Failed to generate uniq_id for url '%s'", url)
			
		path = urlparse(url).path.strip('/')
		return path
	
	def db_find(self, uniq_id: str) -> list[dict]:
		document = None
		ret = []
		try:
			logging.debug("uniq_id to search is '%s'", uniq_id)
			cursor = self.db.find({"uniq_id": uniq_id})
			for document in cursor:
				ret.append({"uniq_id": document["uniq_id"], "tg_file_id": document["tg_file_id"], "media_type": document["media_type"]})
		except Exception as e:
			logging.error("Error occurred while trying to read from the database!")
			logging.exception(e)
		return ret
	
	def db_lookup(self, url: str) -> dict:
		uniq_id = self.compute_uniq(url)
		doc = self.db_find(uniq_id)
		return doc
	
	def db_lookup_id(self, uniq_id: str) -> list[dict]:
		return self.db_find(uniq_id)
	
	def add_media(self, tg_file_ids: list[str], media_url: str, media_type: str, origin: str) -> list[int]:
		uniq_id = self.compute_uniq(media_url)
		media_ids = []
		for tg_file_id in tg_file_ids:
			if not tg_file_id:
				logging.warning("Passed empty `tg_file_id`! Skipping.")
				continue
			if self.db_lookup_id(uniq_id):
				logging.info("Detected existing uniq_id, skipping storage write operation")
				continue
			media_ids += str(self.db.insert_one({"uniq_id": uniq_id, "media_type": media_type, "tg_file_id": tg_file_id, "origin": origin}).inserted_id)

		return media_ids
	
	def get_random(self) -> dict:
		ret = {}
		try:
			cursor = self.db.aggregate([
				{ "$match": { "tg_file_id": { "$exists": True } } },
				{ "$sample": { "size": 1 } }
			])
			tmp = list(cursor)
			if tmp:
				ret = tmp.pop()
		except Exception as e:
			logging.error("Error occurred while trying to read from the database!")
			logging.exception(e)
		return ret






import os, io
import pathlib
import time
import socket
import ssl

from abc import abstractmethod
from typing import Callable, Union

import json

import requests
import urllib
import http.client
import requests
from PIL import Image

from warp_beacon.scraper.abstract import ScraperAbstract
from warp_beacon.mediainfo.abstract import MediaInfoAbstract
from warp_beacon.scraper.exceptions import NotFound, UnknownError, TimeOut, Unavailable, extract_exception_message

from pytubefix import YouTube
from pytubefix.innertube import _default_clients
from pytubefix.streams import Stream
from pytubefix.innertube import InnerTube, _client_id, _client_secret
from pytubefix.exceptions import VideoUnavailable, VideoPrivate, MaxRetriesExceeded
from pytubefix import request

import logging

def patched_fetch_bearer_token(self) -> None:
	"""Fetch an OAuth token."""
	# Subtracting 30 seconds is arbitrary to avoid potential time discrepencies
	start_time = int(time.time() - 30)
	data = {
		'client_id': _client_id,
		'scope': 'https://www.googleapis.com/auth/youtube'
	}
	response = request._execute_request(
		'https://oauth2.googleapis.com/device/code',
		'POST',
		headers={
			'Content-Type': 'application/json'
		},
		data=data
	)
	response_data = json.loads(response.read())
	verification_url = response_data['verification_url']
	user_code = response_data['user_code']

	logging.warning("Please open %s and input code '%s'", verification_url, user_code)
	self.send_message_to_admin_func(
		f"Please open {verification_url} and input code `{user_code}`.\n\n"
		"Please select a Google account with verified age.\n"
		"This will allow you to avoid error the **AgeRestrictedError** when accessing some content.")
	self.auth_event.wait()

	data = {
		'client_id': _client_id,
		'client_secret': _client_secret,
		'device_code': response_data['device_code'],
		'grant_type': 'urn:ietf:params:oauth:grant-type:device_code'
	}
	response = request._execute_request(
		'https://oauth2.googleapis.com/token',
		'POST',
		headers={
			'Content-Type': 'application/json'
		},
		data=data
	)
	response_data = json.loads(response.read())

	self.access_token = response_data['access_token']
	self.refresh_token = response_data['refresh_token']
	self.expires = start_time + response_data['expires_in']
	self.cache_tokens()

class YoutubeAbstract(ScraperAbstract):
	DOWNLOAD_DIR = "/tmp"
	YT_SESSION_FILE = '/var/warp_beacon/yt_session.json'

	def __init__(self) -> None:
		pass

	def __del__(self) -> None:
		pass

	def rename_local_file(self, filename: str) -> str:
		if not os.path.exists(filename):
			raise NameError("No file provided")
		path_info = pathlib.Path(filename)
		ext = path_info.suffix
		old_filename = path_info.stem
		time_name = str(time.time()).replace('.', '_')
		new_filename = "%s%s" % (time_name, ext)
		new_filepath = "%s/%s" % (os.path.dirname(filename), new_filename)

		os.rename(filename, new_filepath)

		return new_filepath

	def remove_tmp_files(self) -> None:
		for i in os.listdir(self.DOWNLOAD_DIR):
			if "yt_download_" in i:
				os.unlink("%s/%s" % (self.DOWNLOAD_DIR, i))

	def download_thumbnail(self, url: str, timeout: int) -> Union[io.BytesIO, None]:
		try:
			reply = requests.get(url, timeout=(timeout, timeout))
			if reply.ok and reply.status_code == 200:
				image = Image.open(io.BytesIO(reply.content))
				image = MediaInfoAbstract.shrink_image_to_fit(image)
				io_buf = io.BytesIO()
				image.save(io_buf, format='JPEG')
				io_buf.seek(0)
				return io_buf
		except Exception as e:
			logging.error("Failed to download download thumbnail!")
			logging.exception(e)

		return None

	def _download_hndlr(self, func: Callable, *args: tuple[str], **kwargs: dict[str]) -> Union[str, dict, io.BytesIO]:
		ret_val = ''
		max_retries = int(os.environ.get("YT_MAX_RETRIES", default=self.YT_MAX_RETRIES_DEFAULT))
		pause_secs = int(os.environ.get("YT_PAUSE_BEFORE_RETRY", default=self.YT_PAUSE_BEFORE_RETRY_DEFAULT))
		timeout = int(os.environ.get("YT_TIMEOUT", default=self.YT_TIMEOUT_DEFAULT))
		timeout_increment = int(os.environ.get("YT_TIMEOUT_INCREMENT", default=self.YT_TIMEOUT_INCREMENT_DEFAULT))
		retries = 0
		while max_retries >= retries:
			try:
				kwargs["timeout"] = timeout
				ret_val = func(*args, **kwargs)
				break
			except MaxRetriesExceeded:
				# do noting, not interested
				pass
			#except http.client.IncompleteRead as e:
			except (socket.timeout, 
					ssl.SSLError, 
					http.client.IncompleteRead, 
					http.client.HTTPException, 
					requests.RequestException, 
					urllib.error.URLError, 
					urllib.error.HTTPError) as e:
				if hasattr(e, "code") and int(e.code) == 403:
					raise Unavailable(extract_exception_message(e))
				logging.warning("Youtube read timeout! Retrying in %d seconds ...", pause_secs)
				logging.info("Your `YT_MAX_RETRIES` values is %d", max_retries)
				logging.exception(extract_exception_message(e))
				if max_retries <= retries:
					self.remove_tmp_files()
					raise TimeOut(extract_exception_message(e))
				retries += 1
				timeout += timeout_increment
				time.sleep(pause_secs)
			except (VideoUnavailable, VideoPrivate) as e:
				raise Unavailable(extract_exception_message(e))

		return ret_val

	def yt_on_progress(self, stream: Stream, chunk: bytes, bytes_remaining: int) -> None:
		pass
		#logging.info("bytes: %d, bytes remaining: %d", chunk, bytes_remaining)

	def build_yt(self, url: str) -> YouTube:
		InnerTube.send_message_to_admin_func = self.send_message_to_admin_func
		InnerTube.auth_event = self.auth_event
		InnerTube.fetch_bearer_token = patched_fetch_bearer_token
		_default_clients["ANDROID"]["innertube_context"]["context"]["client"]["clientVersion"] = "19.08.35"
		_default_clients["ANDROID_MUSIC"] = _default_clients["ANDROID"]
		return YouTube(
			url=url,
			#client="WEB",
			use_oauth=True,
			allow_oauth_cache=True,
			token_file=self.YT_SESSION_FILE,
			on_progress_callback = self.yt_on_progress
		)



import os
import time

import socket
import ssl

from typing import Callable, Optional, Union

from pathlib import Path
import json

import requests
import urllib3
from urllib.parse import urljoin, urlparse

from instagrapi.mixins.story import Story
from instagrapi.types import Media
from instagrapi import Client
from instagrapi.exceptions import LoginRequired, PleaseWaitFewMinutes, MediaNotFound, ClientNotFoundError, UserNotFound, UnknownError as IGUnknownError

from warp_beacon.scraper.exceptions import NotFound, UnknownError, TimeOut, extract_exception_message
from warp_beacon.scraper.abstract import ScraperAbstract
from warp_beacon.jobs.types import JobType
from warp_beacon.telegram.utils import Utils

import logging

INST_SESSION_FILE = "/var/warp_beacon/inst_session.json"

class InstagramScraper(ScraperAbstract):
	cl = None

	def __init__(self) -> None:
		super().__init__()
		self.cl = Client()

	def safe_write_session(self) -> None:
		tmp_fname = "%s~" % INST_SESSION_FILE
		with open(tmp_fname, 'w+') as f:
			f.write(json.dumps(self.cl.get_settings()))
		if os.path.isfile(INST_SESSION_FILE):
			os.unlink(INST_SESSION_FILE)
		os.rename(tmp_fname, INST_SESSION_FILE)

	def load_session(self) -> None:
		if os.path.exists(INST_SESSION_FILE):
			self.cl.load_settings(INST_SESSION_FILE)
		else:
			self.login()

	def login(self) -> None:
		self.cl = Client()
		username = os.environ.get("INSTAGRAM_LOGIN", default=None)
		password = os.environ.get("INSTAGRAM_PASSWORD", default=None)
		verification_code = os.environ.get("INSTAGRAM_VERIFICATION_CODE", default="")
		if username is not None and password is not None:
			self.cl.login(username=username, password=password, verification_code=verification_code)
		self.safe_write_session()

	def scrap(self, url: str) -> tuple[str]:
		self.load_session()
		def _scrap() -> tuple[str]:
			if "stories" in url:
				# remove URL options
				_url = urljoin(url, urlparse(url).path)
				url_last_part = list(filter(None, _url.split('/')))[-1]
				logging.debug("url last part: '%s'", url_last_part)
				if url_last_part.isnumeric():
					return "story", self.scrap_story(url)
				else:
					return "stories", url_last_part
			else:
				return "media", self.scrap_media(url)
		try:
			return _scrap()
		except LoginRequired as e:
			logging.warning("Session error. Trying to relogin...")
			logging.exception(e)
			self.login()
			return _scrap()

	def scrap_stories(self, username: str) -> list[Story]:
		user_info = self.cl.user_info_by_username(username)
		logging.info("user_id is '%s'", user_info.pk)
		return self.cl.user_stories(user_id=user_info.pk)

	def scrap_story(self, url: str) -> str:
		story_id = self.cl.story_pk_from_url(url)
		logging.info("story_id is '%s'", story_id)
		return story_id

	def scrap_media(self, url: str) -> str:
		media_id = self.cl.media_pk_from_url(url)
		logging.info("media_id is '%s'", media_id)
		return media_id
	
	def _download_hndlr(self, func: Callable, *args: tuple[str], **kwargs: dict[str]) -> Union[str, dict]:
		ret_val = {}
		max_retries = int(os.environ.get("IG_MAX_RETRIES", default=5))
		retries = 0
		while max_retries >= retries:
			try:
				ret_val = func(*args, **kwargs)
				break
			except (socket.timeout,
					ssl.SSLError,
					requests.exceptions.ConnectionError,
					requests.exceptions.ReadTimeout,
					requests.exceptions.ConnectTimeout,
					requests.exceptions.HTTPError,
					urllib3.exceptions.ReadTimeoutError,
					urllib3.exceptions.ConnectionError) as e:
				logging.warning("Instagram read timeout! Retrying in 2 seconds ...")
				logging.info("Your `IG_MAX_RETRIES` values is %d", max_retries)
				logging.exception(e)
				if max_retries <= retries:
					raise TimeOut(extract_exception_message(e))
				retries += 1
				time.sleep(2)

		return ret_val

	def download_video(self, url: str, media_info: dict) -> dict:
		path = self._download_hndlr(self.cl.video_download_by_url, url, folder='/tmp')
		return {"local_media_path": str(path), "media_type": JobType.VIDEO, "media_info": {"duration": round(media_info.video_duration)}}

	def download_photo(self, url: str) -> dict:
		path = str(self._download_hndlr(self.cl.photo_download_by_url, url, folder='/tmp'))
		path_lowered = path.lower()
		if ".webp" in path_lowered:
			path = InstagramScraper.convert_webp_to_png(path)
		if ".heic" in path_lowered:
			path = InstagramScraper.convert_heic_to_png(path)
		return {"local_media_path": path, "media_type": JobType.IMAGE}
	
	def download_story(self, story_info: Story) -> dict:
		path, media_type, media_info = "", JobType.UNKNOWN, {}
		logging.info("Story id is '%s'", story_info.id)
		effective_story_id = story_info.id
		if '_' in effective_story_id:
			st_parts = effective_story_id.split('_')
			if len(st_parts) > 1:
				effective_story_id = st_parts[0]
		logging.info("Effective story id is '%s'", effective_story_id)
		effective_url = "https://www.instagram.com/stories/%s/%s/" % (story_info.user.username, effective_story_id)
		if story_info.media_type == 1: # photo
			path = str(self._download_hndlr(self.cl.story_download_by_url, url=story_info.thumbnail_url, folder='/tmp'))
			path_lowered = path.lower()
			if ".webp" in path_lowered:
				path = InstagramScraper.convert_webp_to_png(path)
			if ".heic" in path_lowered:
				path = InstagramScraper.convert_heic_to_png(path)
			media_type = JobType.IMAGE
		elif story_info.media_type == 2: # video
			path = str(self._download_hndlr(self.cl.story_download_by_url, url=story_info.video_url, folder='/tmp'))
			media_type = JobType.VIDEO
			media_info["duration"] = story_info.video_duration

		return {"local_media_path": path, "media_type": media_type, "media_info": media_info, "effective_url": effective_url}

	def download_stories(self, stories: list[Story]) -> dict:
		chunks = []
		for stories_chunk in Utils.chunker(stories, 10):
			chunk = []
			for story in stories_chunk:
				chunk.append(self.download_story(story_info=story))
			chunks.append(chunk)

		return {"media_type": JobType.COLLECTION, "save_items": True, "items": chunks}

	def download_album(self, media_info: dict) -> dict:
		chunks = []
		for media_chunk in Utils.chunker(media_info.resources, 10):
			chunk = []
			for media in media_chunk:
				_media_info = self.cl.media_info(media.pk)
				if media.media_type == 1: # photo
					chunk.append(self.download_photo(url=_media_info.thumbnail_url))
				elif media.media_type == 2: # video
					chunk.append(self.download_video(url=_media_info.video_url, media_info=_media_info))
			chunks.append(chunk)

		return {"media_type": JobType.COLLECTION, "items": chunks}

	def download(self, url: str) -> Optional[list[dict]]:
		res = []
		while True:
			try:
				scrap_type, media_id = self.scrap(url)
				if scrap_type == "media":
					media_info = self._download_hndlr(self.cl.media_info, media_id)
					logging.info("media_type is '%d', product_type is '%s'", media_info.media_type, media_info.product_type)
					if media_info.media_type == 2 and media_info.product_type == "clips": # Reels
						res.append(self.download_video(url=media_info.video_url, media_info=media_info))
					elif media_info.media_type == 1: # Photo
						res.append(self.download_photo(url=media_info.thumbnail_url))
					elif media_info.media_type == 8: # Album
						res.append(self.download_album(media_info=media_info))
				elif scrap_type == "story":
					story_info = self.cl.story_info(media_id)
					logging.info("media_type for story is '%d'", story_info.media_type)
					res.append(self.download_story(story_info=story_info))
				elif scrap_type == "stories":
					logging.info("Stories download mode")
					res.append(self.download_stories(self.scrap_stories(media_id)))
				break
			except PleaseWaitFewMinutes as e:
				logging.warning("Please wait a few minutes error. Trying to relogin ...")
				logging.exception(e)
				wait_timeout = int(os.environ.get("IG_WAIT_TIMEOUT", default=5))
				logging.info("Waiting %d seconds according configuration option `IG_WAIT_TIMEOUT`", wait_timeout)
				if res:
					for i in res:
						if i["media_type"] == JobType.COLLECTION:
							for j in i["items"]:
								if os.path.exists(j["local_media_path"]):
									os.unlink(j["local_media_path"])
						else:
							if os.path.exists(i["local_media_path"]):
								os.unlink(i["local_media_path"])
				os.unlink(INST_SESSION_FILE)
				time.sleep(wait_timeout)
			except (MediaNotFound, ClientNotFoundError, UserNotFound) as e:
				raise NotFound(extract_exception_message(e))
			except IGUnknownError as e:
				raise UnknownError(extract_exception_message(e))
		return res



from warp_beacon.jobs.types import JobType
from warp_beacon.scraper.youtube.abstract import YoutubeAbstract
from warp_beacon.scraper.exceptions import NotFound

from pytubefix import YouTube

import logging

class YoutubeMusicScraper(YoutubeAbstract):
	YT_MAX_RETRIES_DEFAULT = 6
	YT_PAUSE_BEFORE_RETRY_DEFAULT = 3
	YT_TIMEOUT_DEFAULT = 2
	YT_TIMEOUT_INCREMENT_DEFAULT = 60

	def _download(self, url: str, timeout: int = 0) -> list:
		res = []
		thumbnail = None
		yt = self.build_yt(url)

		if yt and yt.thumbnail_url:
			thumbnail = self._download_hndlr(self.download_thumbnail, yt.thumbnail_url)
		
		stream = yt.streams.get_audio_only()
		
		if not stream:
			raise NotFound("No suitable audio stream found")
		
		logging.info("Announced audio file size: '%d'", stream.filesize)
		if stream.filesize > 2e+9:
			logging.warning("Downloading size reported by YouTube is over than 2 GB!")
			raise FileTooBig("YouTube file is larger than 2 GB")
		logging.info("Operation timeout is '%d'", timeout)
		local_file = stream.download(
			output_path=self.DOWNLOAD_DIR,
			max_retries=0,
			timeout=timeout,
			skip_existing=False,
			filename_prefix='yt_download_',
			mp3=True
		)
		logging.debug("Temp filename: '%s'", local_file)
		res.append({
			"local_media_path": self.rename_local_file(local_file),
			"performer": yt.author,
			"thumb": thumbnail,
			"canonical_name": stream.title,
			"media_type": JobType.AUDIO
		})

		return res

	def download(self, url: str) -> list:
		return self._download_hndlr(self._download, url)



from warp_beacon.jobs.types import JobType
from warp_beacon.scraper.youtube.abstract import YoutubeAbstract
from warp_beacon.scraper.exceptions import YotubeLiveError, NotFound, YotubeAgeRestrictedError

from pytubefix import YouTube
from pytubefix.exceptions import AgeRestrictedError

import logging

class YoutubeScraper(YoutubeAbstract):
	YT_MAX_RETRIES_DEFAULT = 8
	YT_PAUSE_BEFORE_RETRY_DEFAULT = 3
	YT_TIMEOUT_DEFAULT = 2
	YT_TIMEOUT_INCREMENT_DEFAULT = 60

	def is_live(self, data: dict) -> bool:
		'''
		x.contents.twoColumnWatchNextResults.results.results.contents[0].videoPrimaryInfoRenderer.viewCount.videoViewCountRenderer.isLive
		'''
		try:
			contents = data.get("contents", {}).get("twoColumnWatchNextResults", {}).get("results", {}).get("results", {}).get("contents", [])
			for i in contents:
				video_view_count_renderer = i.get("videoPrimaryInfoRenderer", {}).get("viewCount", {}).get("videoViewCountRenderer", {})
				if video_view_count_renderer:
					return video_view_count_renderer.get("isLive", False)
		except Exception as e:
			logging.warning("Failed to check if stream is live!")
			logging.exception(e)

		return False

	def _download(self, url: str, timeout: int = 0) -> list:
		res = []
		try:
			thumbnail = None
			yt = self.build_yt(url)

			if self.is_live(yt.initial_data):
				raise YotubeLiveError("Youtube Live is not supported")

			if yt and yt.thumbnail_url:
				thumbnail = self._download_hndlr(self.download_thumbnail, yt.thumbnail_url)

			stream = yt.streams.get_highest_resolution()

			if not stream:
				raise NotFound("No suitable video stream found")

			logging.info("Starting download ...")

			local_file = stream.download(
				output_path=self.DOWNLOAD_DIR,
				max_retries=0,
				timeout=timeout,
				skip_existing=False,
				filename_prefix="yt_download_"
			)
			logging.debug("Temp filename: '%s'", local_file)
			res.append({
				"local_media_path": self.rename_local_file(local_file),
				"performer": yt.author,
				"thumb": thumbnail,
				"canonical_name": stream.title,
				"media_type": JobType.VIDEO
			})
		except AgeRestrictedError as e:
			raise YotubeAgeRestrictedError("Youtube Age Restricted error")

		return res

	def download(self, url: str) -> list:
		return self._download_hndlr(self._download, url)


import os
import pathlib

from abc import ABC, abstractmethod
from typing import Callable, Union

from PIL import Image
from pillow_heif import register_heif_opener

import logging

class ScraperAbstract(ABC):
	send_message_to_admin_func = None
	auth_event = None

	def __init__(self) -> None:
		pass

	def __del__(self) -> None:
		pass

	@abstractmethod
	def download(self, url: str) -> bool:
		raise NotImplementedError

	@abstractmethod
	def _download_hndlr(self, func: Callable, *args: tuple[str], **kwargs: dict[str]) -> Union[str, dict]:
		raise NotImplementedError

	@staticmethod
	def convert_webp_to_png(src_file: str) -> str:
		try:
			if os.path.exists(src_file):
				path_info = pathlib.Path(src_file)
				old_filename = path_info.stem
				new_filename = "%s_converted.%s" % (old_filename, "png")
				new_filepath = "%s/%s" % (os.path.dirname(src_file), new_filename)
				with Image.open(src_file).convert('RGB') as img:
					img.save(new_filepath, 'png')
				os.unlink(src_file)
				return new_filepath
		except Exception as e:
			logging.error("Failed to convert webp file to png!")
			logging.exception(e)

		return ''

	@staticmethod
	def convert_heic_to_png(src_file: str) -> str:
		try:
			if os.path.exists(src_file):
				register_heif_opener()
				path_info = pathlib.Path(src_file)
				old_filename = path_info.stem
				new_filename = "%s_converted.%s" % (old_filename, "png")
				new_filepath = "%s/%s" % (os.path.dirname(src_file), new_filename)
				with Image.open(src_file).convert('RGB') as img:
					img.save(new_filepath, 'png')
				os.unlink(src_file)
				return new_filepath
		except Exception as e:
			logging.error("Failed to convert webp file to png!")
			logging.exception(e)

		return ''



import os, io
import time
from enum import Enum
from typing import Optional

from pyrogram.types import Message
from pyrogram.errors import RPCError, FloodWait
from pyrogram.enums import ParseMode

import warp_beacon
from warp_beacon.telegram.utils import Utils
from warp_beacon.mediainfo.video import VideoInfo

import logging

class PlaceholderType(Enum):
	UNKNOWN = 0
	ANIMATION = 1
	DOCUMENT = 2
	PHOTO = 3

class PlaceHolder(object):
	pl_type: PlaceholderType = PlaceholderType.ANIMATION
	tg_file_id: str = None

	def __init__(self, pl_type: PlaceholderType, tg_file_id: str) -> None:
		self.pl_type = pl_type
		self.tg_file_id = tg_file_id

class PlaceholderMessage(object):
	bot = None
	placeholder = PlaceHolder(PlaceholderType.ANIMATION, None)

	def __init__(self, bot: "Bot") -> None:
		self.bot = bot

	def __del__(self) -> None:
		pass

	async def reuse_ph_animation(self, chat_id: int, reply_id: int, text: str) -> Optional["types.Message"]:
		reply = await self.bot.client.send_animation(
			chat_id=chat_id,
			animation=self.placeholder.tg_file_id,
			caption=text,
			parse_mode=ParseMode.MARKDOWN,
			reply_to_message_id=reply_id
		)

		return reply

	async def reuse_ph_photo(self, chat_id: int, reply_id: int, text: str) -> Optional["types.Message"]:
		reply = await self.bot.client.send_photo(
			chat_id=chat_id,
			photo=self.placeholder.tg_file_id,
			caption=text,
			parse_mode=ParseMode.MARKDOWN,
			reply_to_message_id=reply_id
		)

		return reply

	async def reuse_ph_document(self, chat_id: int, reply_id: int, text: str) -> Optional["types.Message"]:
		reply = await self.bot.client.send_document(
			chat_id=chat_id,
			document=self.placeholder.tg_file_id,
			caption=text,
			parse_mode=ParseMode.MARKDOWN,
			reply_to_message_id=reply_id
		)

		return reply

	async def create(self, chat_id: int, reply_id: int) -> int:
		retry_amount = 0
		max_retries = int(os.environ.get("TG_MAX_RETRIES", default=5))
		while not retry_amount >= max_retries:
			try:
				text = "**Loading, this may take a moment ...** ⏱️ "
				reply = None
				if self.placeholder.tg_file_id is None:
					ph_found = False
					for ph in ('/var/warp_beacon/placeholder.gif', "%s/../var/warp_beacon/placeholder.gif" % os.path.dirname(os.path.abspath(warp_beacon.__file__))):
						if not os.path.exists(ph):
							continue
						try:
							#pl_info = VideoInfo(ph)
							#pl_resolution = pl_info.get_demensions()
							reply = await self.bot.client.send_document(
								chat_id=chat_id,
								document=ph,
								force_document=False,
								caption=text,
								parse_mode=ParseMode.MARKDOWN,
								reply_to_message_id=reply_id,
								file_name=os.path.basename(ph),
								#width=pl_resolution["width"],
								#height=pl_resolution["height"],
								#duration=round(pl_info.get_duration()),
								#thumb=pl_info.generate_thumbnail()
							)
							self.placeholder = PlaceHolder(PlaceholderType.ANIMATION, Utils.extract_file_id(reply))
							ph_found = True
							break
						except Exception as e:
							logging.warning("Failed to send placeholder message!")
							logging.exception(e)
					if not ph_found:
						try:
							reply = await self.bot.client.send_animation(
								chat_id=chat_id,
								animation="https://bagrintsev.me/warp_beacon/placeholder_that_we_deserve.mp4",
								caption=text,
								parse_mode=ParseMode.MARKDOWN,
								reply_to_message_id=reply_id
							)
							self.placeholder = PlaceHolder(PlaceholderType.ANIMATION, Utils.extract_file_id(reply))
						except Exception as e:
							logging.error("Failed to download secret placeholder!")
							logging.exception(e)
							img = self.create_default_placeholder_img("Loading, this may take a moment ...")
							reply = await self.bot.client.send_photo(
								chat_id=chat.id,
								parse_mode=ParseMode.MARKDOWN,
								reply_to_message_id=reply_id,
								photo=img
							)
							self.placeholder = PlaceHolder(PlaceholderType.PHOTO, Utils.extract_file_id(reply))
				else:
					if self.placeholder.pl_type == PlaceholderType.ANIMATION:
						try:
							reply = await self.reuse_ph_animation(chat_id, reply_id, text)
						except ValueError as e:
							logging.warning("Failed to reuse tg_file_id!")
							logging.exception(e)
							reply = await self.reuse_ph_document(chat_id, reply_id, text)
							self.placeholder.pl_type = PlaceholderType.DOCUMENT
					elif self.placeholder.pl_type == PlaceholderType.DOCUMENT:
						try:
							reply = await self.reuse_ph_document(chat_id, reply_id, text)
						except ValueError as e:
							logging.warning("Failed to reuse tg_file_id!")
							logging.exception(e)
							reply = await self.reuse_ph_animation(chat_id, reply_id, text)
							self.placeholder.pl_type = PlaceholderType.ANIMATION
					else:
						reply = await self.reuse_ph_photo(chat_id, reply_id, text)
				return reply.id
			except FloodWait as e:
				logging.warning("FloodWait exception!")
				logging.exception(e)
				await self.bot.send_text(None, "Telegram error: %s" % e.MESSAGE)
				time.sleep(e.value)
			except Exception as e:
				logging.error("Failed to create placeholder message!")
				logging.exception(e)
				retry_amount += 1
				time.sleep(2)

		return 0

	def create_default_placeholder_img(self, text: str, width: int = 800, height: int = 1280) -> io.BytesIO:
		from PIL import Image, ImageDraw, ImageFont
		bio = io.BytesIO()
		bio.name = 'placeholder.png'
		img = Image.new("RGB", (width, height), (255, 255, 255))
		draw = ImageDraw.Draw(img)
		font = ImageFont.load_default(size=48)
		_, _, w, h = draw.textbbox((0, 0), text, font=font)
		draw.text(((width-w)/2, (height-h)/2), text, font=font, fill="#000")
		img.save(bio, 'PNG')
		bio.seek(0)

		return bio

	async def update_text(self, chat_id: int, placeholder_message_id: int, placeholder_text: str) -> None:
		try:
			await self.bot.client.edit_message_caption(
				chat_id=chat_id,
				message_id=placeholder_message_id,
				caption=" ⚠️ **%s**" % placeholder_text,
				parse_mode=ParseMode.MARKDOWN
			)
		except Exception as e:
			logging.error("Failed to update placeholder message!")
			logging.exception(e)

	async def remove(self, chat_id: int, placeholder_message_id: int) -> None:
		try:
			await self.bot.client.delete_messages(chat_id, (placeholder_message_id,))
		except Exception as e:
			logging.error("Failed to remove placeholder message!")
			logging.exception(e)



import codecs
import os.path

from setuptools import setup

def read(rel_path: str) -> str:
	here = os.path.abspath(os.path.dirname(__file__))
	with codecs.open(os.path.join(here, rel_path), 'r') as fp:
		return fp.read()

def get_version(rel_path: str) -> str:
	for line in read(rel_path).splitlines():
		if line.startswith('__version__'):
			delim = '"' if '"' in line else "'"
			return line.split(delim)[1]
	else:
		raise RuntimeError("Unable to find version string.")

setup(
	name="warp_beacon",
	version=get_version("warp_beacon/__version__.py"),
	author="Andrey Bagrintsev",
	author_email="andrey@bagrintsev.me",
	description="Telegram bot for expanding external media links",  # noqa: W605
	include_package_data=True,
	classifiers=[
		'Development Status :: 5 - Production/Stable',

		'Topic :: Sociology',
		'Topic :: Communications :: File Sharing',
		'Topic :: Internet :: WWW/HTTP',
		'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
		'Topic :: Scientific/Engineering :: Image Processing',
		'Topic :: Multimedia :: Video',
		'Topic :: Multimedia',

		'License :: OSI Approved :: Apache Software License',

		# Specify operating system
		'Operating System :: POSIX :: Linux',

		# Specify the Python versions you support here. In particular, ensure
		'Programming Language :: Python :: 3.10'
	],
	license="Apache License",
	url="https://github.com/sb0y/warp_beacon",
	packages=[
		'warp_beacon',
		'warp_beacon/telegram',
		'warp_beacon/uploader',
		'warp_beacon/storage',
		'warp_beacon/scraper',
		'warp_beacon/scraper/youtube',
		'warp_beacon/mediainfo',
		'warp_beacon/jobs',
		'warp_beacon/compress'
	],
	py_modules=[
		"warp_beacon/__version__",
		"warp_beacon/warp_beacon",
		"warp_beacon/telegram/bot",
		"warp_beacon/telegram/placeholder_message",
		"warp_beacon/telegram/handlers",
		"warp_beacon/telegram/utils",
		"warp_beacon/jobs/abstract",
		"warp_beacon/jobs/download_job",
		"warp_beacon/jobs/upload_job",
		"warp_beacon/mediainfo/abstract",
		"warp_beacon/mediainfo/video",
		"warp_beacon/mediainfo/audio",
		"warp_beacon/mediainfo/silencer",
		"warp_beacon/compress/video",
		"warp_beacon/scraper/abstract",
		"warp_beacon/scraper/exceptions",
		"warp_beacon/scraper/types",
		"warp_beacon/scraper/instagram",
		"warp_beacon/scraper/youtube/abstract",
		"warp_beacon/scraper/youtube/youtube",
		"warp_beacon/scraper/youtube/shorts",
		"warp_beacon/scraper/youtube/music"
	],
	#scripts=['scripts/wait_dc_update.py'],
	data_files=[
		("/lib/systemd/system/",
			["etc/warp_beacon.service"]
		),
		("/etc/warp_beacon/",
			["etc/warp_beacon.conf"]
		),
		("/var/warp_beacon/",
			["assets/placeholder.gif"]
		)
	],

	#entry_points={
	#	'console_scripts': [
	#		'warp_beacon = warp_beacon.warp_beacon:main'
	#	]
	#}
)




#import numpy as np
import av

from warp_beacon.mediainfo.video import VideoInfo

import logging

class Silencer(VideoInfo):
	def add_silent_audio(self) -> str:
		try:
			new_filepath = self.generate_filepath(self.filename)
			if self.container:
				in_video_stream = next(s for s in self.container.streams if s.type == 'video')
				codec_name = in_video_stream.codec_context.name
				fps = in_video_stream.base_rate
				#time_base = in_video_stream.time_base
				#duration = float(in_video_stream.duration * time_base)
				with av.open(new_filepath, 'w') as out_container:
					out_video_stream = out_container.add_stream(codec_name, rate=fps)
					out_audio_stream = out_container.add_stream('aac')
					out_video_stream.width = in_video_stream.codec_context.width
					out_video_stream.height = in_video_stream.codec_context.height
					out_video_stream.pix_fmt = in_video_stream.codec_context.pix_fmt
					out_video_stream.time_base = in_video_stream.time_base
					for frame in self.container.decode(in_video_stream):
						packet = out_video_stream.encode(frame)
						if packet:
							out_container.mux(packet)
							#
							aframe = av.AudioFrame(samples=64, format='s16')
							aframe.pts = frame.pts
							aframe.sample_rate = 16000
							aframe.rate = 44100
							for packet in out_audio_stream.encode(aframe):
								out_container.mux(packet)

					remain_packets = out_video_stream.encode(None)
					out_container.mux(remain_packets)
				self.filename = new_filepath
				return new_filepath
		except Exception as e:
			logging.error("Error occurred while generating silenced video file!")
			logging.exception(e)

		return ''